module Model_Parameters
        
        implicit none
        
        real(8), parameter :: eta   = 1d0/0.6d0                         ! Inverse Frisch elasticity
        real(8), parameter :: mu   = 0.02d0                             ! The growth rate of the economy
        real(8), parameter :: beta   = 1.00399d0                       !Discount factor
        integer, parameter :: Tret   = 36                               ! Years in retirement  (65-100)
        integer, parameter :: T      = 45                               ! Years of active life (20-64)
        real(8), parameter :: tc    = 0.05d0                            ! Consumption tax
        real(8), parameter :: t_employer=0.0765d0, t_employee=0.0765d0  !Social security tax paid by employer and employee
        real(8), parameter :: tk     = 0.36d0                           ! Capital tax 
        real(8), parameter :: sigma  = 4d0                              ! Risk aversion parameter
        real(8), parameter :: alpha  = 1d0/3d0                          ! Capital share
        real(8), parameter :: delta  = 0.0988d0-mu                      ! Capital depreciation
        real(8), parameter :: chi= 22.69d0 
        ! Disutility from work
        real(8), parameter :: sigma_u=0.30661466d0, rho_u=0.3353638d0   ! Parameters governing the process for the  idiosyncratic shock, men
        real(8), parameter :: sigma_a=0.34012837d0, rho_a=0d0           ! Stdev of ability, men
        real(8) :: gammaw(3)                                            ! Returns to experience parameters
        real(8) :: AE                                                   ! Average earnings
        real(8) :: theta(2)                                             ! Labor tax parameters
        real(8) :: tax_level_scale=0.9591d0, tax_prog_scale=1.0d0          ! Parameters to scale tax- level and progressivity
        real(8), parameter :: ybar = 1.2d0                              ! Upper limit for SS tax
        real(8) :: epsilon=1d0, epsilon2=1d0, epsilon3=1d0, epsilon4=1d0, epsilon5=1d0, epsilon6=1d0
        integer, parameter :: na     = 5                                ! Number of permanent ability levels
        integer, parameter :: nu     = 5                                ! Number of transitory productivity shocks
        integer, parameter :: nc     = 100                              ! Number of gridpoints in "static" hours function
        integer, parameter :: nw     = 100                              ! Number of gridpoints in "static" hours function
        integer, parameter :: nk     = 16                               ! Number of gridpoints over capital
        integer, parameter :: nsim   = 10000                            ! Number of households used for simulation
        integer, parameter :: nsim2   = 16                              ! Number of simulations
        
        real(8), dimension (:), allocatable :: a, Prob_a              ! Vectors with the value of ability and the probability of each ability
        real(8), dimension (:), allocatable :: u, Prob_u              ! Vectors with the value of the idiosyncratic shock and the probability of each shock
        real(8), dimension (:,:), allocatable :: trans_u              ! Transition matrix for the shock u
        real(8), dimension (:,:), allocatable :: trans_a              ! Dummy transition matrix for a (used to compute the unconditional probability of each a)
        
        real(8) :: Gamma_redistr = 0.01928014399460030d0
        real(8) :: Psi_pension = 0.295319951736185d0
        real(8) :: lumpsum=0.09825562269736179d0, lumpsumdum=0.09825562269736179d0
        
        real(8), dimension (:), allocatable :: OmegaRet, OmegaActive
        real(8), dimension (:), allocatable :: WeightRet, WeightActive
        real(8) :: ratio=4.28989876546317d0, ratiodum=4.28989876546317    !Ratio between capital and labor
        real(8) :: r                                    !Real interest rate
        real(8) :: debttoGDP=0.6185043538d0             ! Government debt as % of GDP
        real(8) :: milspendtoGDP=0.03625d0              ! Military spanding as % of GDP
        real(8) :: w                                    ! Wages per efficiency unit
        
        real(8), dimension (:,:,:,:), allocatable :: Sim1 ! Array to hold simulation results

        integer, dimension (:,:,:,:), allocatable :: exp1 ! Array to hold simulation results
        
        real(8), dimension (:,:,:,:), allocatable :: SimR1 ! Array to hold simulation results for retired
        real(8), dimension (:,:,:), allocatable :: Random3
        real(8), dimension (:,:), allocatable :: Random1, Random2
        integer :: it,iter
           
    end module Model_Parameters