import numpy as np
import pandas as pd
import os
import matplotlib.pyplot as plt
import re
import simdata_processing_routines as sdp

# Without soc. security taxes:
tn_base = 0.174748980734487
# tn_base = 0.169097593407509
# tn_base = 0.1725
tk_base = 0.36

gc_data = {}
gc_taxrev_max = {}

re_float = '[\-]?\d+\.\d+[eE]?[+\-]?\d+'

pattern_list = {'Level'           : r'^ level =\s+({0})'.format(re_float), \
'Average labor income tax rate'   : r'^ Average labor income tax rate is\s+({0})'.format(re_float), \
'Labor income tax rate with TSS'  : r'^ Labor income tax rate including TSS is\s+({0})'.format(re_float), \
'Tax revenue per capita'          : r'^ Tax revenue per capita is\s+({0})'.format(re_float), \
'Tax revenue per capita with TSS' : r'^ Tax revenue per capita including TSS is\s+({0})'.format(re_float), \
'Labor supply'                    : r'^ Labor supply is\s+({0})'.format(re_float), \
'Average individ. earnings'       : r'^ Average individual earnings is\s+({0})'.format(re_float), \
'Wage'                            : r'^ Wage is\s+({0})'.format(re_float), \
'Female LFP'                      : r'^ Female labor force participation is\s+({0})'.format(re_float), \
'Capital tax per capita'          : r'^ Capital tax per capita is\s+({0})'.format(re_float), \
'Consumption tax per capita'      : r'^ Consumption tax per capita is\s+({0})'.format(re_float), \
'Labor income tax per capita'     : r'^ Labor income tax per capita is\s+({0})'.format(re_float), \
'Single female LFP'               : r'^ Single female labor force participation is\s+({0})'.format(re_float), \
'Married female LFP'              : r'^ Married female labor force participation is\s+({0})'.format(re_float), \
'GDP per capita':  r'^ GDP per capita is\s+({0})'.format(re_float), \
'Interest rate': r'^ Interest is\s+({0})'.format(re_float)
}

for proglev in ['0', '1', '2', '3', 'dk']:
    file_gc = 'prog_{:s}.txt'.format(proglev)
    gc_data[proglev] = sdp.find_patterns_pd(file_gc, pattern_list)
    gc_data[proglev]['Efficiency labor units'] = \
            gc_data[proglev]['Average individ. earnings']/gc_data[proglev]['Wage']
    gc_data[proglev]['Tax revenue per capita adj'] = \
            gc_data[proglev]['Tax revenue per capita'] - gc_data[proglev]['GDP per capita']*0.6185*gc_data[proglev]['Interest rate']*tk_base

for item in ['Tax revenue per capita', \
        'Tax revenue per capita adj', \
        'Efficiency labor units', \
        'Average individ. earnings', \
        'Female LFP', 'Capital tax per capita', \
        'Consumption tax per capita', \
        'Labor income tax per capita']:
    adj_factor = np.interp(tn_base, gc_data['1']['Average labor income tax rate'], gc_data['1'][item])
    for proglev in gc_data:
        gc_data[proglev][item] = gc_data[proglev][item]/adj_factor*100.0


plot_specs = {'0': ['k', '-', None, 4, 'k', 'Flat tax', [4,4,4,4]], \
              '1': ['k', '-', None, None, None, '1 x US prog. ', []], \
              '2': ['k', '-', None, None, None, '2 x US prog. ', [8,4,2,4,2,4]], \
              '3': ['0.65', '-', None, None, None, '3 x US prog. ', [4,4,4,4]], \
              'dk': ['k',   ':', None, None, None, 'DK prog. ', []]}


us_bench = [tn_base, 100.0]

fig = plt.figure()
ax = plt.subplot(111)

for proglev in ['0', '1', '2', '3']:
    plot_cond = ( gc_data[proglev]['Average labor income tax rate'] > 0.12 )
    ax.plot(gc_data[proglev]['Average labor income tax rate'][plot_cond], 
            gc_data[proglev]['Tax revenue per capita'][plot_cond], 
            color = plot_specs[proglev][0], 
            linestyle = plot_specs[proglev][1],
            marker = plot_specs[proglev][2], 
            markersize = plot_specs[proglev][3], 
            markeredgecolor = plot_specs[proglev][4], 
            label = plot_specs[proglev][5],
            dashes = plot_specs[proglev][6])
    (max_x, max_y) = sdp.find_maxval(gc_data[proglev]['Average labor income tax rate'], 
            gc_data[proglev]['Tax revenue per capita'],detailed=True)
    print 'Progressivity: ' + proglev
    print max_x, max_y

ax.plot(gc_data['dk']['Average labor income tax rate'][plot_cond], 
        gc_data['dk']['Tax revenue per capita'][plot_cond], 
        color = 'k', 
        linestyle = ':',
        label = 'DK prog. ')
(max_x, max_y) = sdp.find_maxval(gc_data['dk']['Average labor income tax rate'], 
        gc_data['dk']['Tax revenue per capita adj'],detailed=True)
print 'Progressivity: dk'
print max_x, max_y


ax.plot(us_bench[0], us_bench[1], 'D', color='0.75')
ax.set_xlim([0.1, 0.8])
ax.set_ylim([80, 180])
plt.xlabel('Labor income tax rate', fontsize=12)
plt.ylabel('Tax revenue, % of benchmark', fontsize=12)
box = ax.get_position()
ax.set_position([box.x0, box.y0, box.width * 0.8, box.height])
leg = ax.legend(bbox_to_anchor=(1,0.7), ncol=1, prop={'size':10})
leg.get_frame().set_alpha(0.0)

plt.ion()
plt.show()

