#!/usr/bin/env python

def find_patterns(file_in, pattern_list, bnd_pat):
    """
    Goes through file `file_in', and picks lines that correspond
    to patterns in pattern_list; all numbers in the matches between the boundaries
    defined by the pattern in bnd_pat are recorded as separate lines
    in the resulting matrix, which is then returned;
    patterns in pattern_list must contain at least one number, only the first number
    in the pattern is stored
    """

    import re
    import numpy as np
    import os.path
    import sys

    npat = len(pattern_list)

    if os.path.isfile(file_in):
        f = open(file_in,'r')
        fdat = f.readlines()
        f.close()
        
        rawdat = []

        pt = np.zeros(npat)
        matchfound = np.zeros(npat)
        
        il = 0
        nm = 0

        for line in fdat:
            il += 1
            s0 = 1  # s0 = 1: search for boundary pattern, s0 = 0: do not search for boundary pattern

            for ip in range(npat):
                if matchfound[ip] == 0:
                    pat = pattern_list[ip]
                    m = re.search(pat, line)
                    if m:
                        pt[ip] = float(m.group(1))
                        matchfound[ip] = 1
                        s0 = 0
                        break

            if s0:
                m0 = re.search(bnd_pat, line)
                if m0:
                    #if (nm > 0):
                        #rawdat.append(pt)
                    rawdat.append(pt)
                    pt = np.zeros(npat)
                    matchfound = np.zeros(npat)
                    nm += 1

        if sum(matchfound) > 0:
            rawdat.append(pt)

        npdat = np.array(rawdat)
        return npdat

    else:
        print('File {0:s} does not exist in current folder'.format(file_in))
        sys.exit()

def find_patterns2(file_in, pattern_list):
    """
    Goes through file `file_in', and picks lines that correspond
    to patterns in pattern_list; 
    patterns in pattern_list must contain at least one number, only the first number
    in the pattern is stored
    """

    import re
    import numpy as np
    import os.path
    import sys

    npat = len(pattern_list)

    if os.path.isfile(file_in):
        f = open(file_in,'r')
        fdat = f.readlines()
        f.close()
        
        rawdat = []

        pt = np.zeros(npat)
        matchfound = np.zeros(npat)
        
        il = 0
        nm = 0

        for line in fdat:
            il += 1
            #print line

            for ip, pat in enumerate(pattern_list):
                if not matchfound[ip]:
                    m = re.search(pat, line)
                    if m:
                        pt[ip] = float(m.group(1))
                        matchfound[ip] = 1
                        break

            if (sum(matchfound) == npat):
                rawdat.append(pt)
                pt = np.zeros(npat)
                matchfound = np.zeros(npat)
                
        npdat = np.array(rawdat)
        return npdat

    else:
        print('File {0:s} does not exist in current folder'.format(file_in))
        sys.exit()

def find_patterns_pd(file_in, pattern_list):
    """
    Goes through file `file_in', finds lines that correspond
    to patterns in pattern_list, collects corresponding data
    and returns them as a Panda DataFrame
    """

    import re
    import numpy as np
    import pandas as pd
    import os.path
    import sys

    my_keys = pattern_list.keys()
    my_keys.remove('Level')
    df = pd.DataFrame(columns=my_keys)
    search_keys = []
    num_pat = len(my_keys)
    if os.path.isfile(file_in):
        f = open(file_in, 'r')
        fdat = f.readlines()
        f.close()
        for lineno, line in enumerate(fdat):
            #print line
            m = re.search(pattern_list['Level'], line)
            if m:
                tax_level = m.group(1)
                df.loc[tax_level] = [0.0]*num_pat
                search_keys = my_keys[:]
            else:
                for key in search_keys:
                    pattern = pattern_list[key]
                    m = re.search(pattern, line)
                    if m:
                        df.ix[tax_level, key] = float(m.group(1))
                        search_keys.remove(key)
        return df
    else:
        print('File {0:s} does not exist in current folder'.format(file_in))
        sys.exit()

def find_maxval(x, y, detailed=False):
    import numpy as np
    if detailed == True:
        ndetailed = 1000
        tmp = np.zeros((ndetailed,2))
        lo = x[0]
        hi = x[-1]
        tmp[:,0] = np.linspace(lo, hi, ndetailed)
        for k in range(ndetailed):
            tmp[k,1] = np.interp(tmp[k,0], x, y)
    else:
        tmp = np.zeros((len(x),2))
        tmp[:,0] = x 
        tmp[:,1] = y

    max_y = max(tmp[:,1])
    max_x = tmp[np.where( tmp[:,1] == max_y)[0][0], 0]
    return (max_x, max_y)

def test1(filename):
    bnd = r'Tax function parameters:'
    pt_lst = [ r'^ level =\s+(\d+\.\d+[eE]?[+\-]?\d+)', \
            r'^ Average labor income tax rate is\s+(\d+\.\d+[eE]?[+\-]?\d+)', \
            r'^ Labor income tax rate including TSS is\s+(\d+\.\d+[eE]?[+\-]?\d+)', \
            r'^ Tax revenue per capita including TSS is\s+(\d+\.\d+[eE]?[+\-]?\d+)' ]

    #res = find_patterns("C:\Users\stepanch\Dropbox\Laffer_Curves (2) (1)\Results\Sensitivity_to_utility_function\g-curves\Life_cycle_crra_prog0.0.txt", pt_lst, bnd)
    res = find_patterns(filename, pt_lst, bnd)
    res2 = find_patterns2(filename, pt_lst)

    print(res)

def process_file(filenamein, filenameout, dtype):
    """
    dtype = 1: (x = tax rate with TSS, y = tax revenue per capital with TSS)
    dtype = 2: (x = tax rate without TSS, y = tax revenues without TSS)
    dtype = 3: (x = tax rate without TSS, y = debt per capita)
    dtype = 4: (x = tax rate with TSS, y = labor supply)

    """

    import os.path
    import sys

    if os.path.isfile(filenamein):
        # Define patterns to match:
        pat1a = r'Labor income tax rate including TSS is\s+(\d+\.\d+[eE]?[+\-]?\d+)'
        pat1b = r'Average labor income tax rate is\s+(\d+\.\d+[eE]?[+\-]?\d+)'
        pat1c = r'Average labor income tax rate is\s+(\d+\.\d+[eE]?[+\-]?\d+)'
        pat2a = r'Tax revenue per capita including TSS is\s+(\d+\.\d+[eE]?[+\-]?\d+)'
        pat2b = r'Tax revenue per capita is\s+(\d+\.\d+[eE]?[+\-]?\d+)'
        pat2c = r'Debt per capita is\s+(\d+\.\d+[eE]?[+\-]?\d+)'
        pat2d = r'Labor supply is\s+(\d+\.\d+[eE]?[+\-]?\d+)'
        pat2f_part1 = r'Average earnings is\s+(\d+\.\d+[eE]?[+\-]?\d+)'
        pat2f_part2 = r'Wage is\s+(\d+\.\d+[eE]?[+\-]?\d+)'

        pat0 = r'Progressivity =\s+(\d+\.\d+[eE]?[+\-]?\d+)'

        pdat = []

        import re
        import numpy as np

        # Process the file:
        f = open(filenamein,'r')
        fdat = f.readlines()
        f.close()
        
        s0 = 1
        progressivity = 0
        if (dtype == 1):
            s1 = 1
            s2 = 0
        elif (dtype in [2, 3, 4]):
            s1 = 0
            s2 = 1
        elif (dtype == 5):
            s2 = 1
            part1 = 1
            s1 = 0

        i = 1
        pt = [0.0, 0.0]

        for line in fdat:
            i = i + 1
            if (s0):
                m0 = re.search(pat0,line)
                if m0:
                    progressivity = float(m0.group(1))
                    s0 = 0

            if (s1):
                if (dtype == 1):
                    m1 = re.search(pat1a,line)
                elif (dtype == 2):
                    m1 = re.search(pat1b,line)
                elif (dtype == 3):
                    m1 = re.search(pat1c,line)
                elif (dtype == 4):
                    m1 = re.search(pat1a,line)
                elif (dtype == 5):
                    m1 = re.search(pat1a,line)

                if m1:
                    pt[0] = float(m1.group(1))
                    s1 = 0
                    s2 = 1
                    if (dtype == 2 or dtype == 3 or dtype == 4):
                        pdat.append(pt)
                        pt = [0.0, 0.0]
                    
            else:
                if (dtype == 1):
                    m2 = re.search(pat2a,line)
                elif (dtype == 2):
                    m2 = re.search(pat2b,line)
                elif (dtype == 3):
                    m2 = re.search(pat2c,line)
                elif (dtype == 4):
                    m2 = re.search(pat2d,line)
                elif (dtype == 5 and part1 == 1):
                    m2 = re.search(pat2f_part1,line)
                elif (dtype == 5 and part1 == 0):
                    m2 = re.search(pat2f_part2,line)

                if (m2 and dtype < 5):
                    s2 = 0
                    s1 = 1
                    pt[1] = float(m2.group(1))
                    if (dtype == 1):
                        pdat.append(pt)
                        pt = [0.0, 0.0]
                elif (m2 and dtype == 5 and part1 == 1):
                    pt[1] = float(m2.group(1))
                    part1 = 0
                    s2 = 0
                    s1 = 1
                elif (m2 and dtype == 5 and part1 == 0):
                    pt[1] = pt[1]/float(m2.group(1))
                    pdat.append(pt)
                    pt = [0.0, 0.0]
                    part1 = 1

        n = len(pdat)

        pdat_np = np.array(pdat)
        

        # If one needs to remove the first observation,
        # since in some programs it was computed with an error:
        tmp = np.delete(pdat_np, 0, 0)
        pdat_np = tmp
        n = n-1


        order = pdat_np[:,0].argsort()
        sorted = np.take(pdat_np, order, 0)
        pdat_np = sorted
        x = pdat_np[:,0]
        y = pdat_np[:,1]


        f = open(filenameout,'w')
        f.write('{0:12.7f} {1:12.7f}\n'.format(progressivity, 0.0))
        for i in range(n):
            f.write('{0:12.7f}  {1:12.7f}\n'.format(x[i], y[i]))
        f.close()

    else:
        print('File {0:s} does not exist in current folder'.format(filename))
        sys.exit()


if __name__ == "__main__":
    # # Get all command-line arguments:
    # import argparse
    # parser = argparse.ArgumentParser()

    # parser.add_argument("-i", "--filein", type=str,
            # required=True, help="choose the data file to be processed")

    # parser.add_argument("-o", "--fileout", type=str, 
            # required=True, help="choose the output file")

    # parser.add_argument("-d", "--detailed_type", type=int,
            # choices = [1,2,3], required=False, default=2, 
            # help="choose between laffer curves with TSS (1), without TSS (2) and debt curve (3)")

    # args = parser.parse_args()
    # filenamein = args.filein
    # filenameout = args.fileout
    # dtype = args.detailed_type
    # process_file(filenamein, filenameout, dtype)
    
    # For debugging:
    #process_file("C:\Users\stepanch\Dropbox\Laffer_Curves (2) (1)\Results\Sensitivity_to_utility_function\g-curves\Life_cycle_crra_prog0.0.txt", "tmp", 5)

    # For debugging find_pattern:
    test1('../International_Laffer_Curves/g-curves/Germany_wages.txt')
